!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                   XEFFORT LIBRARY SAMPLES
!=======================================================================
! XSciCalc.f90 - Main application module for XSciCalc sample
! 
! XSciCalc is a simple dialog-based application which launches Notepad
! so that user can edit an input file, does a long calculation, and
! spawns an "output window".
!
! XSciCalc sample demonstrates new capabilities of Xeffort 1.2.17:
! * XProcessMessages routine 
! * using XCtlSet(...0) to append text to multi-line edit boxes
! as well as 
! * using owner-drawn static controls 
!
! 2005. Xeffort.com
!        Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================
MODULE XSciCalc

USE Xeffort
USE DFWIN

USE OutWindow

IMPLICIT NONE

INCLUDE 'Resource.fd'

!========================================================================
CONTAINS
!========================================================================
!PURPOSE: Initialization function called by XFT library on app start.
LOGICAL FUNCTION XInit(szCmdLine,nCmdShow)
!DEC$IF (_DF_VERSION_.GE.650)
!DEC$ATTRIBUTES DEFAULT, DECORATE, ALIAS: 'XINIT':: XInit
!DEC$ELSE
!DEC$ATTRIBUTES ALIAS: '_XINIT@12':: XInit
!DEC$ENDIF

IMPLICIT NONE

CHARACTER(*), INTENT(IN)::    szCmdLine
INTEGER, INTENT(IN)::         nCmdShow

LOGICAL::            bSt

IF (XCreateDialogApp(IDD_MAIN, idIcon=IDI_ICON_MAIN)) THEN
   !Todo: Initialize dialog controls here via XCtlSet(Sub)
   bSt = XCtlSetSub(XW_FRAME, IDC_EDIT, XW_FRAME_OnEdit)
   bSt = XCtlSet(XW_FRAME, IDC_CALCULATE, .FALSE., CTL_ENABLE)
   bSt = XCtlSetSub(XW_FRAME, IDC_CALCULATE, XW_FRAME_OnCalculate)
   IF (XModalDialog(XW_FRAME).EQ.IDOK) THEN
      !Todo: place logic after dialog is exited via OK button here (XCtlGet)
   END IF
END IF

!For a dialog-bassed application, the XInit may return .FALSE.
!Alternatively, use XModelessDialog, but return .TRUE. and set
!handlers for IDOK/IDCANCEL buttons.
XInit = .FALSE.

END FUNCTION XInit
!=======================================================================
!Callback for "Edit" button. Launches Notepad.exe and waits for it to complete.
SUBROUTINE XW_FRAME_OnEdit(xWnd, idCmd, nCode)
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd 
INTEGER, INTENT(IN):: nCode 

TYPE(T_STARTUPINFO):: SI
TYPE(T_PROCESS_INFORMATION):: PI
INTEGER::                     iSt
LOGICAL::            bSt

!Disable UI elements
bSt = XCtlSet(xWnd, IDC_EDIT, .FALSE., CTL_ENABLE)
bSt = XCtlSet(xWnd, IDC_CALCULATE, .FALSE., CTL_ENABLE)

!Launch Notepad.
!NB: it's preferred to use ShellExecuteEx instead of CreateProcess, because
!that will launch user's preferred editor for .txt files. Do you like Notepad?
!Me not.
SI%cb = SIZEOF(SI)
SI%dwFlags = 0
iSt = CreateProcess(NULL, 'Notepad.exe XSciCalc.txt'C, NULL, NULL, FALSE, 0, NULL, NULL, SI, PI)

!Wait for Notepad to finish and process messages meanwhile
iSt = XProcessMessages(.TRUE., hSyncEvents=(/PI%hProcess/))

iSt = CloseHandle(PI%hProcess)
iSt = CloseHandle(PI%hThread)

!Enable UI elements back.
bSt = XCtlSet(xWnd, IDC_EDIT, .TRUE., CTL_ENABLE)
bSt = XCtlSet(xWnd, IDC_CALCULATE, .TRUE., CTL_ENABLE)

END SUBROUTINE XW_FRAME_OnEdit
!=======================================================================
!Callback for "Calculate" button.
SUBROUTINE XW_FRAME_OnCalculate(xWnd, idCmd, nCode)
 
TYPE(X_WINDOW)::      xWnd
INTEGER, INTENT(IN):: idCmd 
INTEGER, INTENT(IN):: nCode 

LOGICAL::             bSt
INTEGER::             iSt, i, j, k, iErr, nPoints

!Disable UI elements
bSt = XCtlSet(xWnd, IDC_EDIT, .FALSE., CTL_ENABLE)
bSt = XCtlSet(xWnd, IDC_CALCULATE, .FALSE., CTL_ENABLE)

!Create & show the output window
CALL CreateOutputWindow()

!Read data:
OPEN(11, FILE="XSciCalc.txt")
nPoints = 0
DO i=1,1000
   READ(11,*,IOSTAT=iErr) m_xPoint(i)
   IF (iErr.EQ.0) THEN
      nPoints = nPoints+1
   ELSE
      EXIT
   END IF
END DO
CLOSE(11)

!Set up the log window & progress bar
CALL xOutWnd_WriteText("Calculation started...")
iSt = XCtlSet(xWnd, IDC_PROGRESS1, nPoints, CTL_RANGEMAX)

!Run the calculation:
m_yPoint(0) = 0.
m_yMin = 1000
m_yMax = -1000
DO m_nPoints = 1,nPoints
      i = m_nPoints
   
      m_yPoint(i) = 0. 
      DO j=1,1000
         m_yPoint(i) = m_yPoint(i) + j * MAX(1., m_xPoint(i))**(-j)
         !This is a no-op loop used to slow things down. Reduce
         !the loop count to a smaller value if it's too slow for you.
         DO k=1,10000
            m_yPoint(i) = m_yPoint(i) + 1-2*MOD(k,2)
         END DO
      END DO
      m_yMin = MIN(m_yMin, m_YPoint(i))
      m_yMax = MAX(m_yMax, m_YPoint(i))

      !Note that xOutWnd may be closed by user while the calculation is
      !running. Check whether it exists first:
      IF (XIsWindow(xOutWnd, WINDOW)) THEN
         !Update log window
         CALL xOutWnd_WriteText(CHAR(13)//CHAR(10)//"Evaluating point "// &
                                XString(i)//", x = "//XString(m_XPoint(i),2)//" y = "//XString(m_YPoint(i),6))
         !Update entire xOutWnd. That will update its child windows as well.
         CALL XUpdateWindow(xOutWnd)
      END IF
      bSt = XCtlSet(xWnd, IDC_PROGRESS1, m_nPoints, CTL_STATE)
      
      !Call XProcessMessages to enable proper redrawing and user input
      iSt = XProcessMessages()
      IF (iSt.EQ.WM_QUIT) RETURN
END DO

!Wait for xOutWnd to be closed by user before proceeding:
IF (XIsWindow(xOutWnd, WINDOW)) iSt = XProcessMessages(.TRUE., (/WM_OUTWINDOWCLOSED/))

!Everything done, enable UI buttons.
bSt = XCtlSet(xWnd, IDC_EDIT, .TRUE., CTL_ENABLE)
bSt = XCtlSet(xWnd, IDC_CALCULATE, .TRUE., CTL_ENABLE)


END SUBROUTINE XW_FRAME_OnCalculate
!========================================================================
END MODULE XSciCalc

